<?php

class CrontabPrestaProducts 
{
    // Constructeur
    public function __construct()
    {
        /* Disable WP theme for this file (optional) */
        define('WP_USE_THEMES', false);
        
        /** Sets up the WordPress Environment. */
        require __DIR__ . '/../../../../../wp-load.php';
        
        global $wpdb;
        
        echo __( 'Traitement de mise à jour du cache.', 'presta-products' ) . PHP_EOL;
        echo sprintf(__( '[%s] Début du traitement.', 'presta-products' ), date('d/m/Y H:i:s')) . PHP_EOL;
                
        // Réinit la base
        $sql = $wpdb->prepare("TRUNCATE TABLE {$wpdb->prefix}ppgbo_cache") or die('error');
        $wpdb->query($sql);
        
        // Récupération des articles et pages
        $posts = get_posts(array('posts_per_page' => -1));
        $pages = get_pages();
        
        // Parcours pour obtenir les shortcodes
        foreach (array_merge($posts, $pages) as $post) {
            // Si le shortcode est trouvé
            if (preg_match_all('/^(.*)(\[ppgbo[a-zA-Z0-9=",\s]+\])(.*)$/misU', $post->post_content, $matched)) {
                // Parcours des données
                foreach ($matched[2] as $match) {
                    // Initialisation
                    $params     = array();
                    $products   = array();
                    $datas      = "";
                    
                    // Récupération des paramètres
                    $attributes = explode(' ', $match);
                    unset($attributes[0]);
                    
                    // Parcours des données
                    foreach ($attributes as $attribute) {
                        $param              = explode('=', $attribute);
                        $params[$param[0]]  = preg_replace('/]/', '', preg_replace('/"/', '', $param[1]));
                    }
                    
                    // Récupération des produits
                    $datas = ppgbo_get_products_from_prestashop($params);
                    
                    foreach ($datas as $data) {
                        $products[] = array('url' => $data->url_rewrite, 'name' => $data->product->name);
                    }
                    
                    // Requête SQL d'ajout
                    $sql = $wpdb->prepare("INSERT IGNORE INTO {$wpdb->prefix}ppgbo_cache (shortcode, datas, products) VALUES ('%s', '%s', '%s')", json_encode($params), json_encode($datas), json_encode($products));
                    $wpdb->query($sql) or wp_die(__( 'Problème d\'enregistrement de la donnée.', 'presta-products' ));
                }
            }
        }
        
        echo sprintf(__( '[%s] Fin du traitement.', 'presta-products' ), date('d/m/Y H:i:s')) . PHP_EOL;
        
        // Fonction de récupération des produits depuis le site prestashop
        function ppgbo_get_products_from_prestashop($atts) {
            $prestashop_url     = get_option( 'ppgbo_prestashop_url' );
            $webservice_key     = get_option( 'ppgbo_prestashop_webservice' );
            $hide_inactive      = get_option( 'ppgbo_hide_inactive' );
            $hide_available     = get_option( 'ppgbo_hide_available' );
            $hide_visibility    = get_option( 'ppgbo_hide_visibility' );
            
            // Initialisation du tableau des produits
            $products = array();
    
            // 1 produit
            if (isset($atts['product'])) {
                $url[] = $prestashop_url . '/api/products/' . $atts['product'] . '?price[my_price][use_tax]=1&price[old_price][only_reduction]=1&ws_key=' . $webservice_key . '&output_format=JSON';
            }
            // Plusieurs produits
            else if (isset($atts['products'])) {
                $ids = explode(',', $atts['products']);
                
                foreach ($ids as $id) {
                    $url[] = $prestashop_url . '/api/products/' . $id . '?price[my_price][use_tax]=1&price[old_price][only_reduction]=1&ws_key=' . $webservice_key . '&output_format=JSON';
                }
            }
            else if (isset($atts['bestsellers'])) {
                // Nombre de jours en arrière
                $days   = (isset($atts['bestsellers']) && intval($atts['bestsellers'])) ? $atts['bestsellers'] : 10;
                $today  = date('Y-m-d');
                $back   = date('Y-m-d', time() - ($days * 24 * 60 * 60));
                
                // Appel HTTP API
                $link       = $prestashop_url . '/api/orders/?filter[date_add]=[' . $back . ',' . $today . ']&display=full&date=1&ws_key=' . $webservice_key . '&output_format=JSON';
                $response   = wp_remote_get( $link );
                $body       = wp_remote_retrieve_body( $response );
                $http_code  = wp_remote_retrieve_response_code( $response );
                
                if ($http_code == '200') {
                    // Produit (format JSON > Array)
                    $result     = json_decode($body);
                    $_products  = array();
    
                    // Parcours des lignes
                    foreach ($result->orders as $element) {
                        foreach ($element->associations->order_rows as $row) {
                            $_products[$row->product_id] = intval($_products[$row->product_id]) + 1;
                        }
                    }
                    
                    // Fonctions de traitement du tableau
                    arsort($_products);
                    $_products = array_keys($_products);
                    array_splice($_products, $limit);
    
                    // Récupération produit
                    foreach ($_products as $element) {
                        $url[] = $prestashop_url . '/api/products/' . $element . '?price[my_price][use_tax]=1&price[old_price][only_reduction]=1&ws_key=' . $webservice_key . '&output_format=JSON';
                    }
                }
            }
            // Catégorie
            else if (isset($atts['category'])) {
                // Appel HTTP API
                $link       = $prestashop_url . '/api/products/?filter[id_category_default]=[' . $atts['category'] . ']&limit=' . $limit . '&ws_key=' . $webservice_key . '&output_format=JSON';
                $response   = wp_remote_get( $link );
                $body       = wp_remote_retrieve_body( $response );
                $http_code  = wp_remote_retrieve_response_code( $response );
            
                if ($http_code == '200') {
                    // Produit (format JSON > Array)
                    $result = json_decode($body);
    
                    foreach ($result->products as $element) {
                        $url[] = $prestashop_url . '/api/products/' . $element->id . '?price[my_price][use_tax]=1&price[old_price][only_reduction]=1&ws_key=' . $webservice_key . '&output_format=JSON';
                    }
                }
            }
            // Plusieurs catégories
            else if (isset($atts['categories'])) {
                $ids = explode(',', $atts['categories']);
                
                foreach ($ids as $id) {
                    // Appel HTTP API
                    $link       = $prestashop_url . '/api/products/?filter[id_category_default]=[' . $id . ']&limit=' . $limit . '&ws_key=' . $webservice_key . '&output_format=JSON';
                    $response   = wp_remote_get( $link );
                    $body       = wp_remote_retrieve_body( $response );
                    $http_code  = wp_remote_retrieve_response_code( $response );
                
                    if ($http_code == '200') {
                        // Produit (format JSON > Array)
                        $result = json_decode($body);
    
                        foreach ($result->products as $element) {
                            $url[] = $prestashop_url . '/api/products/' . $element->id . '?price[my_price][use_tax]=1&price[old_price][only_reduction]=1&ws_key=' . $webservice_key . '&output_format=JSON';
                        }
                    }
                }
            }
            
            // Récupération des produits
            foreach ($url as $link) {
                // Appel HTTP API
                $response   = wp_remote_get( $link );
                $body       = wp_remote_retrieve_body( $response );
                $http_code  = wp_remote_retrieve_response_code( $response );
            
                if ($http_code == '200') {
                    // Produit (format JSON > Array)
                    $product = json_decode($body);
                    
                    // Si le produit est trouvé, on l'ajoute au tableau des produits
                    if (isset($product->product)) {
                        // Si on veut cacher des produits
                        if ($product->product->active == 0 && $hide_inactive) {
                            continue;
                        }
                        else if ($product->product->available_for_order == 0 && $hide_available) {
                            continue;
                        }
                        else if ($product->product->visibility == 'none' && $hide_visibility) {
                            continue;
                        }
                        
                        // Génération de l'URL produit
                        $url_rewrite            = $prestashop_url . '/index.php?controller=product&id_product=' . $product->product->id;
                        $url_rewrite_element    = wp_remote_head($url_rewrite);
                        
                        // Récupération de la redirection par les méthodes WP
    					if (isset($url_rewrite_element['headers']) && isset($url_rewrite_element['headers']['location'])) {
    					    $product->url_rewrite = $url_rewrite_element['headers']['location'];
    					}
    					
    					// Récupération de la redirection par les fonctions PHP (si la précédente n'a pas marché)
    					if (empty($product->url_rewrite) || !isset($product->url_rewrite)) {
    						$product->url_rewrite = get_headers($url_rewrite, 1)['Location'];
    					}
    					
                        // URL par défaut (si aucune redirection n'est trouvée)
    					if (empty($product->url_rewrite) || !isset($product->url_rewrite)) {
    					    if (is_array($product->product->link_rewrite) && count($product->product->link_rewrite) > 0) {
    					        $link_rewrite = $product->product->link_rewrite[0]->value;
    					    }
    					    else {
    					        $link_rewrite = $product->product->link_rewrite;
    					    }
    					    
    						$product->url_rewrite = $prestashop_url . '/' . $product->product->id . ((isset($product->product->id_default_combination) && !empty($product->product->id_default_combination)) ? '-' . $product->product->id_default_combination : '' ) . '-' . $link_rewrite .'.html';
    					}
    					
    					// Gestion de la langue
    					$id_lang = 1;
    					
    					if (is_array($product->product->name) && count($product->product->name) > 0) {
    						foreach ($product->product->name as $key => $name) { 
    						    if ($name->id == $id_lang) { 
    						        $keyToSave = $key; 
    						        break;
    						    } 
    						}
    						
    						$product->product->name                 = $product->product->name[$keyToSave]->value;
    						$product->product->description          = $product->product->description[$keyToSave]->value;
    						$product->product->description_short    = $product->product->description_short[$keyToSave]->value;
    					}
    					
                        $products[] = $product;
                    }
                }
            }
            
            return $products;
        }
    }
}

// Appel de la classe
new CrontabPrestaProducts();